@@ -6,16 +6,19 @@ import stat
 import subprocess
 import tempfile
 from pathlib import Path
-from typing import Any, List
+from typing import Any, List, Optional
 
 from selenium import webdriver
 from webdrivermanager import AVAILABLE_DRIVERS
 
+from RPA.core.types import is_list_like
+
+
 LOGGER = logging.getLogger(__name__)
 
 DRIVER_DIR = Path(tempfile.gettempdir()) / "drivers"
 DRIVER_PREFERENCE = {
-    "Windows": ["Chrome", "Firefox", "Edge", "IE", "Opera"],
+    "Windows": ["Chrome", "Firefox", "Edge", "Ie", "Opera"],
     "Linux": ["Chrome", "Firefox", "Opera"],
     "Darwin": ["Chrome", "Safari", "Firefox", "Opera"],
     "default": ["Chrome", "Firefox"],
@@ -51,23 +54,27 @@ CHROME_VERSION_COMMANDS = {
 }
 
 
-def executable(browser: str, download: bool = False) -> str:
+def executable(browser: str, download: bool = False) -> Optional[str]:
     """Get path to webdriver executable, and download it if requested.
 
     :param browser: name of browser to get webdriver for
     :param download: download driver binaries if they don't exist
     """
     LOGGER.debug(
-        "Webdriver initialization for '%s' (download: %s)", browser, download,
+        "Webdriver initialization for '%s' (download: %s)",
+        browser,
+        download,
     )
 
     browser = browser.lower().strip()
     factory = AVAILABLE_DRIVERS.get(browser)
-
     if not factory:
         return None
 
     driver_path = _driver_path(factory, download)
+    if driver_path is None:
+        LOGGER.debug("Failed to get driver path for %s", browser)
+        return None
 
     if driver_path.exists() and not download:
         LOGGER.debug("Attempting to use existing driver: %s", driver_path)
@@ -97,31 +104,44 @@ def start(name: str, **options):
     name = name.strip()
     try:
         factory = getattr(webdriver, name)
-    except AttributeError:
-        raise RuntimeError(f"'{name}' is not a valid driver name")
+    except AttributeError as e:
+        raise RuntimeError(f"'{name}' is not a valid driver name") from e
 
     driver = factory(**options)
     return driver
 
 
-def _driver_path(factory: Any, download: bool) -> Any:
+def _driver_path(factory: Any, download: bool) -> Optional[Path]:
     if platform.system() != "Windows":
         manager = factory(link_path="/usr/bin")
     else:
         manager = factory()
 
-    filename = manager.get_driver_filename()
+    driver_names = manager.get_driver_filename()
 
-    temp_path = Path(DRIVER_DIR) / filename
-    link_path = Path(manager.link_path) / filename
+    if driver_names is None:
+        return None
 
-    if temp_path.exists() or download:
-        return temp_path
-    else:
-        return link_path
+    if not is_list_like(driver_names):
+        driver_names = [driver_names]
+
+    primary_path = Path(DRIVER_DIR) / driver_names[0]
+    if download or primary_path.exists():
+        return primary_path
+
+    for name in driver_names:
+        temp_path = Path(DRIVER_DIR) / name
+        if temp_path.exists():
+            return temp_path
+
+        link_path = Path(manager.link_path) / name
+        if link_path.exists():
+            return link_path
+
+    return Path(manager.link_path) / driver_names[0]
 
 
-def _chrome_version() -> str:
+def _chrome_version() -> Optional[str]:
     system = platform.system()
     commands = CHROME_VERSION_COMMANDS.get(system)
 
@@ -143,7 +163,7 @@ def _chrome_version() -> str:
     return None
 
 
-def _chromedriver_version(path: Path) -> str:
+def _chromedriver_version(path: Path) -> Optional[str]:
     output = _run_command([str(path), "--version"])
     if not output:
         return None
@@ -170,20 +190,23 @@ def _download_driver(factory: Any, version: str = None) -> None:
             _set_executable_permissions(bin_path)
 
         LOGGER.debug(
-            "%s downloaded to %s", manager.get_driver_filename(), bin_path,
+            "%s downloaded to %s",
+            manager.get_driver_filename(),
+            bin_path,
         )
-    except RuntimeError:
-        pass
+    except Exception as exc:
+        raise RuntimeError(f"Failed to download wedriver: {exc}") from exc
 
 
 def _set_executable_permissions(path: str) -> None:
     st = os.stat(path)
     os.chmod(
-        path, st.st_mode | stat.S_IXOTH | stat.S_IXGRP | stat.S_IEXEC,
+        path,
+        st.st_mode | stat.S_IXOTH | stat.S_IXGRP | stat.S_IEXEC,
     )
 
 
-def _run_command(args: List[str]) -> str:
+def _run_command(args: List[str]) -> Optional[str]:
     try:
         output = subprocess.check_output(args)
         return output.decode().strip()
